@@ -51,7 +51,7 @@ hill.climbing = function(x, start, whitelist, blacklist, score,
     start$learning$ntests = 0
     start$learning$test = score
     start$learning$args = extra.args
-    start$learning$optimized = TRUE
+    start$learning$optimized = optimized
 
   }#THEN
 
@@ -251,5 +251,5 @@ hill.climbing = function(x, start, whitelist, blacklist, score,
 
   return(start)
 
-}#HILL.CLIMBING.OPTIMIZED
+}#HILL.CLIMBING
 
