@@ -140,7 +140,7 @@ inter.ia.markov.blanket = function(x, data, nodes, alpha, B, whitelist, blacklis
   if (debug) {
 
     cat("----------------------------------------------------------------\n")
-    cat("* detecting markov blanket of", x, ".\n")
+    cat("* learning markov blanket of", x, ".\n")
 
   }#THEN
 
